@@ -1,4 +1,4 @@
-/* $Id: thunar-vfs-path-private.h 24219 2006-12-30 12:20:31Z benny $ */
+/* $Id: thunar-vfs-path-private.h 26421 2007-12-02 13:46:28Z benny $ */
 /*-
  * Copyright (c) 2005-2006 Benedikt Meurer <benny@xfce.org>
  *
@@ -30,12 +30,16 @@
 G_BEGIN_DECLS;
 
 /* Support macros for compilers that don't support proper inlining */
+#if !defined(G_CAN_INLINE) && !defined(__THUNAR_VFS_PATH_C__) && !defined(__THUNAR_VFS_INFO_C__)
+
 #define thunar_vfs_path_ref(path)         (exo_atomic_inc (&(THUNAR_VFS_PATH ((path))->ref_count)), path)
 #define thunar_vfs_path_is_root(path)     (THUNAR_VFS_PATH ((path))->parent == NULL)
 #define thunar_vfs_path_get_name(path)    (((const gchar *) path) + sizeof (ThunarVfsPath))
 #define thunar_vfs_path_get_parent(path)  (THUNAR_VFS_PATH ((path))->parent)
 #define thunar_vfs_path_get_scheme(path)  (THUNAR_VFS_PATH ((path))->ref_count & THUNAR_VFS_PATH_SCHEME_MASK)
 
+#endif /* !defined(G_CAN_INLINE) && !defined(__THUNAR_VFS_PATH_C__) && !defined(__THUNAR_VFS_INFO_C__) */
+
 /* global shared variables */
 extern ThunarVfsPath *_thunar_vfs_path_trash_root G_GNUC_INTERNAL;
 
