@@ -1,6 +1,6 @@
-/* $Id: thunar-util.h 22648 2006-08-04 12:03:37Z benny $ */
+/* $Id: thunar-util.h 26421 2007-12-02 13:46:28Z benny $ */
 /*-
- * Copyright (c) 2006 Benedikt Meurer <benny@xfce.org>
+ * Copyright (c) 2006-2007 Benedikt Meurer <benny@xfce.org>
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -22,11 +22,14 @@
 
 #include <thunar-vfs/thunar-vfs.h>
 
+#include <thunar/thunar-enum-types.h>
+
 G_BEGIN_DECLS;
 
 gboolean   thunar_util_looks_like_an_uri  (const gchar      *string) G_GNUC_INTERNAL G_GNUC_WARN_UNUSED_RESULT;
 
-gchar     *thunar_util_humanize_file_time (ThunarVfsFileTime file_time) G_GNUC_INTERNAL G_GNUC_MALLOC G_GNUC_WARN_UNUSED_RESULT;
+gchar     *thunar_util_humanize_file_time (ThunarVfsFileTime file_time,
+                                           ThunarDateStyle   date_style) G_GNUC_INTERNAL G_GNUC_MALLOC G_GNUC_WARN_UNUSED_RESULT;
 
 GdkScreen *thunar_util_parse_parent       (gpointer          parent,
                                            GtkWindow       **window_return) G_GNUC_INTERNAL G_GNUC_WARN_UNUSED_RESULT;
